/**
 * @function appServer
 */

'use strict'

const Koa = require('koa')
const KoaRouter = require('koa-router')
const http = require('http')
const co = require('co')

/** @lends appServer */
function appServer (endpoints) {
  let router = new KoaRouter()
  let app = new Koa()
  let server = http.createServer(app.callback())

  app
    .use(co.wrap(function * setServer (ctx, next) {
      ctx.server = server
      yield next()
    }))
    .use(co.wrap(function * uncaughtErrorHandler (ctx, next) {
      try {
        yield next()
      } catch (err) {
        console.error('Uncaught exception', err)
        ctx.status = 500
      }
    }))

  for (let endpoint of [].concat(endpoints)) {
    let { pathname, handler } = endpoint
    for (let h of [].concat(handler)) {
      let hasPathname = pathname && pathname !== '*'
      if (hasPathname) {
        router.use(pathname, h)
      } else {
        app.use(h)
      }
    }
    router.all(pathname, co.wrap(function * (ctx, next) {
      yield next()
    }))
  }
  app
    .use(router.routes())
    .use(router.allowedMethods())

  return server
}

module.exports = appServer
