/**
 * @memberof module:apeman-app
 * @function getApps
 * @param {Apemanfile} configuration
 * @param {string[]} names
 */

'use strict'

const textconv = require('textconv')

const debug = require('debug')('apeman:app')

let concat = (a, b) => [].concat(a, b)

/** @lends getApps */
function getApps (configuration, names) {
  return [].concat(names)
    .reduce(concat, [])
    .map((name) => {
      debug(`Load app: "${name}"`)
      if (/#/.test(name)) {
        return getApps.findChildApps(configuration, name)
      } else {
        return configuration.get(`$apps.${name}`)
      }
    })
    .reduce(concat, [])
    .map((app) => {
      switch (typeof app) {
        case 'string':
          return getApps(configuration, app)
        default:
          return app
      }
    })
    .reduce(concat, [])
}

Object.assign(getApps, {
  findChildApps (configuration, name) {
    let namepaths = name.split(/#/g).map(textconv.trim)
    let namespace = namepaths.shift()
    let $child = configuration.$children[ namespace ]
    if (!$child) {
      throw new Error('$child not found with name: ' + name)
    }
    let apps = namepaths.length === 1
      ? $child.get(`$apps.${namepaths.shift()}`)
      : getApps.findChildApps($child, namepaths.join('#'))
    return [].concat(apps || []).map((app) => {
      if (typeof app === 'string') {
        return getApps(configuration, [ namespace, app ].join('#'))
      } else {
        return app
      }
    }).reduce(concat, [])
  }
})

module.exports = getApps
