/**
 * List available apps.
 * @memberof module:apeman-app
 * @function listApps
 * @param {Apemanfile} configuration - Apemanfile to work with.
 * @param {object} options - Optional settings.
 * @returns {Promise}
 */

'use strict'

const argx = require('argx')
const ListLogger = require('../logging/list_logger')
const Promise = require('apemanpromise')

/** @lends listApps */
function listApps (configuration, options) {
  let args = argx(arguments)
  configuration = args.shift()
  options = args.pop('object') || {}
  let apps = configuration.get('$apps')
  let $children = configuration.get('$children') || {}
  let childApps = Object.keys($children).map((name) => {
    let $child = $children[ name ]
    return {
      namespace: name,
      apps: $child.get('$apps') || {},
      cwd: $child.$cwd
    }
  })
  let logger = new ListLogger({
    verbose: options.verbose
  })
  logger.logList(apps, childApps)
  return Promise.resolve(null)
}

module.exports = listApps
