/**
 * Test case for appServer.
 * Runs with mocha.
 */
'use strict'

const appServer = require('../lib/apping/app_server.js')
const apemanport = require('apemanport')
const apemanrequest = require('apemanrequest')
const injectmock = require('injectmock')
const co = require('co')
const assert = require('assert')

describe('app-server', () => {
  let request = apemanrequest.create({ jar: true })
  before(() => co(function * () {
    injectmock(console, 'error', () => {})
  }))

  after(() => co(function * () {
    injectmock.restoreAll()
  }))

  it('App server', () => co(function * () {
    let port = yield apemanport.find()
    let server = appServer([
      {
        pathname: '',
        handler: require('apeman-app-route')({
          '/users/:user_id': {
            GET: co.wrap(function * handler (ctx) {
              throw new Error('hogehoge')
            })
          }
        })
      }
    ])
    server.listen(port)

    // Handle unhandled error
    {
      let res = yield request({
        method: 'GET',
        url: `http://localhost:${port}/users/1`
      })
      let { body, statusCode } = res
      assert.equal(statusCode, 500)
    }
    server.close()
  }))
})

/* global describe, before, after, it */
